import { CellType, FileExtensionType, ExcelFileOptions } from './types';
export default class ExcelFile {
    readonly name: string;
    readonly data: CellType[][];
    readonly options?: ExcelFileOptions | undefined;
    private wb;
    constructor(name: string, data: CellType[][], options?: ExcelFileOptions | undefined);
    /**
     * Check name validation
     * It throws error when name is null or empty
     * @param name
     */
    private validateName;
    /**
     * Return fileName with date time at the end of the name
     */
    private addDateTimeToName;
    /**
     * Check data validation
     * It throws error when data is empty or has different row length
     * @param data
     */
    private validateData;
    /**
     * Return formatted data
     * @param data
     */
    private formatData;
    /**
     * Create workSheet from provided data
     */
    private createWorkSheet;
    /**
     * Create workBook with only one sheet.
     * Appended sheet will be named 'Sheet1' for default.
     */
    private createWorkBook;
    /**
     * Format file full name by joinning name and extension
     */
    private formatFullName;
    /**
     * Download xlsx or csv file
     * @param extension file extension, default is 'xlsx'
     */
    download(extension?: FileExtensionType): void;
}
